@@ -1,6 +1,6 @@
 /* atmequal.c - Compares ATM addresses for equality */
 
-/* Written 1995-1999 by Werner Almesberger, EPFL-LRC/ICA */
+/* Written 1995-2000 by Werner Almesberger, EPFL-LRC/ICA */
 
 
 #include <string.h>
@@ -35,7 +35,7 @@ static int atm_equal_svc(const struct sockaddr_atmsvc *a,
 	a_prv = a->sas_addr.prv;
 	b_prv = b->sas_addr.prv;
 	if ((flags & AXE_WILDCARD) && len >= 8 && *a_prv == ATM_AFI_E164 &&
-	  *b_prv == ATM_AFI_E164)
+	  *b_prv == ATM_AFI_E164) {
 	    if (len < 68) return 0; /* no comparison possible */
 	    else {
 		int a_pos,b_pos;
@@ -57,6 +57,7 @@ static int atm_equal_svc(const struct sockaddr_atmsvc *a,
 		b_prv += 9;
 		if ((len -= 72) < 0) len = 0;
 	    }
+	}
 	if (memcmp(a_prv,b_prv,len/8)) return 0;
 	if ((len & 7) && (a_prv[len/8+1]^b_prv[len/8+1]) &
 	  (0xff00 >> (len & 7))) return 0;
