@@ -1,6 +1,6 @@
 /* saaldump.c - ATM signaling message dumper */
 
-/* Written 1997,1998 by Werner Almesberger, EPFL-LRC/ICA */
+/* Written 1997-1999 by Werner Almesberger, EPFL-LRC/ICA */
 
 
 #include <stdlib.h>
@@ -19,7 +19,12 @@
 #include "qlib.h"
 
 
+#define MAX_ITEM  2048 /* longest string emitted by q.dump */
+
+
 static int interval = 0; /* display absolute time by default */
+static int quiet = 0; /* decode Q.2931 messages by default */
+static int new_line = 1;
 
 
 static void sscop_pdu_diag(int severity,const char *fmt,...)
@@ -36,13 +41,21 @@ static void sscop_pdu_diag(int severity,const char *fmt,...)
 
 void qd_dump(const char *msg,...)
 {
+    char buf[MAX_ITEM];
     va_list ap;
+    int len;
 
+    if (new_line) printf("    ");
     va_start(ap,msg);
-    printf("    ");
-    vprintf(msg,ap);
-    putchar('\n');
+    len = vsprintf(buf,msg,ap);
     va_end(ap);
+    if (len > MAX_ITEM) {
+	fprintf(stderr,"FATAL ERROR: buffer too small (%d < %d)\n",MAX_ITEM,
+	  len);
+	exit(1);
+    }
+    printf("%s",buf);
+    new_line = len ? buf[len-1] == '\n' : 0;
 }
 
 
@@ -51,11 +64,12 @@ void qd_report(int severity,const char *msg,...)
     va_list ap;
 
     if (severity > Q_ERROR) return;
+    printf("%s    ERROR: ",new_line ? "" : "\n");
     va_start(ap,msg);
-    printf("    ERROR: ");
     vprintf(msg,ap);
-    putchar('\n');
     va_end(ap);
+    putchar('\n');
+    new_line = 1;
 }
 
 static void analyze(unsigned char *pdu,int size,struct timeval stamp)
@@ -65,7 +79,7 @@ static void analyze(unsigned char *pdu,int size,struct timeval stamp)
     PDU_VARS;
 
     if (first || !interval) {
-	printf("%2d:%02d:%02d",(int) ((stamp.tv_sec/3600) % 24),
+	printf("%2d:%02d:%02d ",(int) ((stamp.tv_sec/3600) % 24),
 	  (int) ((stamp.tv_sec/60) % 60),(int) (stamp.tv_sec % 60));
 	if (interval) {
 	    first = 0;
@@ -82,11 +96,12 @@ static void analyze(unsigned char *pdu,int size,struct timeval stamp)
 	    diff.tv_sec--;
 	}
 	last = stamp;
-	printf("%8u",(unsigned) diff.tv_sec);
+	printf("%05u.%02u",(unsigned) diff.tv_sec,
+	  (unsigned) diff.tv_usec/10000);
     }
     if (!DECOMPOSE_PDU(NULL,pdu,size)) {
-	PRINT_PDU("  ",pdu);
-	if (type == SSCOP_SD) {
+	PRINT_PDU("",pdu);
+	if (type == SSCOP_SD && !quiet) {
 	    Q_DSC dsc;
 
 	    qd_open(&dsc,pdu,length);
@@ -98,7 +113,7 @@ static void analyze(unsigned char *pdu,int size,struct timeval stamp)
 
 static void usage(const char *name)
 {
-    fprintf(stderr,"usage: %s [-i] [itf.]vpi.vci\n",name);
+    fprintf(stderr,"usage: %s [-i] [-q] [itf.]vpi.vci\n",name);
     exit(1);
 }
 
@@ -112,11 +127,14 @@ int main(int argc,char **argv)
     int c,s,size;
 
     name = argv[0];
-    while ((c = getopt(argc,argv,"i")) != EOF)
+    while ((c = getopt(argc,argv,"iq")) != EOF)
 	switch (c) {
 	    case 'i':
 		interval = 1;
 		break;
+	    case 'q':
+		quiet = 1;
+		break;
 	    default:
 		usage(name);
 	}
