@@ -1,6 +1,6 @@
 /* io.c - I/O operations */
  
-/* Written 1995-1999 by Werner Almesberger, EPFL-LRC/ICA */
+/* Written 1995-2000 by Werner Almesberger, EPFL-LRC/ICA */
  
 
 #include <stdlib.h>
@@ -13,11 +13,11 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
-#include <linux/atmdev.h>
-#include <stdint.h> /* linux/atmsvc.h includes linux/atm.h */
-#include <linux/atmsvc.h>
 
 #include "atm.h"
+#include <linux/atmdev.h>
+#include <linux/atmsvc.h>
+
 #include "atmd.h"
 #include "uni.h"
 #include "pdu.h"
@@ -93,9 +93,9 @@ void to_kernel(struct atmsvc_msg *msg)
 {
     int wrote;
 
-    diag("KERNEL",DIAG_DEBUG,"TO KERNEL: %s (%d) for 0x%lx/0x%lx",
-      as_name[msg->type],msg->reply,(unsigned long) msg->vcc,
-      (unsigned long) msg->listen_vcc);
+    diag("KERNEL",DIAG_DEBUG,"TO KERNEL: %s (%d) for %s/%s",
+      as_name[msg->type],msg->reply,kptr_print(&msg->vcc),
+      kptr_print(&msg->listen_vcc));
 	/* should be "IO" ... */
     trace_kernel("TO KERNEL",msg);
     wrote = write(kernel,msg,sizeof(*msg));
