@@ -1,17 +1,17 @@
 /* kernel.c - Processing of incoming kernel messages */
  
-/* Written 1995-1999 by Werner Almesberger, EPFL-LRC/ICA */
+/* Written 1995-2000 by Werner Almesberger, EPFL-LRC/ICA */
  
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
 #include <assert.h>
-#include <stdint.h> /* linux/atmsvc.h includes linux/atm.h */
+
+#include "atm.h"
 #include <linux/atmsvc.h>
 #include <linux/atmdev.h>
 
-#include "atm.h"
 #include "atmd.h"
 #include "uni.h"
 #include "qlib.h"
@@ -66,7 +66,7 @@ static int send_setup(SOCKET *sock)
 	    }
 	}
     }
-    if (local)
+    if (local) {
 	if (*local->sas_addr.pub) {
 	    q_assign(&dsc,QF_cgpn_plan,ATM_NP_E164);
 	    q_assign(&dsc,QF_cgpn_type,ATM_TON_INTRNTNL);
@@ -78,6 +78,7 @@ static int send_setup(SOCKET *sock)
 	    q_assign(&dsc,QF_cgpn_type,ATM_TON_UNKNOWN);
 	    q_write(&dsc,QF_cgpn,(void *) local->sas_addr.prv,ATM_ESA_LEN);
 	}
+    }
     q_assign(&dsc,QF_call_ref,call_ref);
     q_assign(&dsc,QF_msg_type,ATM_MSG_SETUP);
     q_assign(&dsc,QF_aal_type,5); /* AAL 5 */
@@ -201,14 +202,15 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		if (!get_local(NULL)) SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
 		else
 #endif
-		send_kernel(msg->vcc,0,as_okay,0,NULL,NULL,get_local(NULL),
-		  NULL,NULL);
+		send_kernel(msg->vcc,kptr_null,as_okay,0,NULL,NULL,
+		  get_local(NULL),NULL,NULL);
 	    else {
 		if (!route_local(&msg->svc)) {
 		    SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
 		    return;
 		}
-		send_kernel(msg->vcc,0,as_okay,0,NULL,NULL,NULL,NULL,NULL);
+		send_kernel(msg->vcc,kptr_null,as_okay,0,NULL,NULL,NULL,NULL,
+		  NULL);
 	    }
 	    return;
 	case as_connect: /* NULL state only */
@@ -291,7 +293,7 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 	    sock->local = msg->svc;
 	    sock->sap = msg->sap;
 	    sock->qos = msg->qos;
-	    send_kernel(sock->id,0,as_okay,0,NULL,NULL,NULL,NULL,NULL);
+	    send_kernel(sock->id,kptr_null,as_okay,0,NULL,NULL,NULL,NULL,NULL);
 	    sock->state = ss_listening;
 	    return;
 	case as_close: /* all but INDICATED, PROCEEDING, ZOMBIE, and WAIT_REL */
@@ -376,33 +378,33 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 #if defined(Q2963_1) || defined(DYNAMIC_UNI)
 	case as_modify:
 	    if (sock && !(sock->sig->uni & S_Q2963_1)) {
-		send_kernel(sock->id,0,as_okay,-ENOPROTOOPT,NULL,NULL,NULL,
-		  NULL,NULL);
+		send_kernel(sock->id,kptr_null,as_okay,-ENOPROTOOPT,NULL,NULL,
+		  NULL,NULL,NULL);
 		return;
 	    }
 	    if (sock && (sock->state == ss_mod_lcl ||
 	      sock->state == ss_mod_req || sock->state == ss_mod_rcv ||
 	      sock->state == ss_mod_fin_ok || sock->state == ss_mod_fin_fail ||
 	      sock->state == ss_mod_fin_ack)) {
-		send_kernel(sock->id,0,as_okay,-EALREADY,NULL,NULL,NULL,NULL,
-		  NULL);
+		send_kernel(sock->id,kptr_null,as_okay,-EALREADY,NULL,NULL,
+		  NULL,NULL,NULL);
 		return;
 	    }
 	    if (!sock || sock->state != ss_connected || !sock->owner) {
-		send_kernel(sock->id,0,as_okay,-EBADFD,NULL,NULL,NULL,NULL,
-		  NULL);
+		send_kernel(sock->id,kptr_null,as_okay,-EBADFD,NULL,NULL,NULL,
+		  NULL,NULL);
 		return;
 	    }
 	    if (sock->qos.txtp.traffic_class != msg->qos.txtp.traffic_class ||
 	      sock->qos.rxtp.traffic_class != msg->qos.rxtp.traffic_class) {
 	/* @@@ may do more checking */
-		send_kernel(sock->id,0,as_okay,-EINVAL,NULL,NULL,NULL,NULL,
-		  NULL);
+		send_kernel(sock->id,kptr_null,as_okay,-EINVAL,NULL,NULL,NULL,
+		  NULL,NULL);
 		return;
 	    }
 	    sock->new_qos = msg->qos;
-	    send_kernel(sock->id,0,as_modify,ATM_MF_INC_RSV | ATM_MF_DEC_SHP,
-	      NULL,NULL,NULL,NULL,&msg->qos);
+	    send_kernel(sock->id,kptr_null,as_modify,
+	      ATM_MF_INC_RSV | ATM_MF_DEC_SHP,NULL,NULL,NULL,NULL,&msg->qos);
 	    new_state(sock,ss_mod_lcl);
 	    return;
 	case as_okay:
@@ -418,12 +420,13 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		    new_state(sock,ss_connected);
 		    return;
 		case ss_mod_fin_ok:
-		    send_kernel(sock->id,0,as_okay,0,NULL,NULL,NULL,NULL,NULL);
+		    send_kernel(sock->id,kptr_null,as_okay,0,NULL,NULL,NULL,
+		      NULL,NULL);
 		    new_state(sock,ss_connected);
 		    return;
 		case ss_mod_fin_fail:
-		    send_kernel(sock->id,0,as_okay,sock->error,NULL,NULL,NULL,
-		      NULL,NULL);
+		    send_kernel(sock->id,kptr_null,as_okay,sock->error,NULL,
+		      NULL,NULL,NULL,NULL);
 		    sock->error = 0;
 		    /* fall through */
 		case ss_mod_fin_ack:
@@ -435,8 +438,8 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 	case as_error:
 	    switch (sock ? sock->state : ss_null) {
 		case ss_mod_lcl:
-		    send_kernel(sock->id,0,as_okay,msg->reply,NULL,NULL,NULL,
-		      NULL,NULL);
+		    send_kernel(sock->id,kptr_null,as_okay,msg->reply,NULL,
+		      NULL,NULL,NULL,NULL);
 		    new_state(sock,ss_connected);
 		    return;
 		case ss_mod_rcv:
@@ -445,14 +448,15 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		    return;
 		case ss_mod_fin_ok:
 		    diag(COMPONENT,DIAG_ERROR,"QOS commit failed");
-		    send_kernel(sock->id,0,as_okay,0,NULL,NULL,NULL,NULL,NULL);
+		    send_kernel(sock->id,kptr_null,as_okay,0,NULL,NULL,NULL,
+		      NULL,NULL);
 		    /* @@@ clear call instead ? */
 		    new_state(sock,ss_connected);
 		    return;
 		case ss_mod_fin_fail:
 		    diag(COMPONENT,DIAG_ERROR,"QOS rollback failed");
-		    send_kernel(sock->id,0,as_okay,sock->error,NULL,NULL,NULL,
-		      NULL,NULL);
+		    send_kernel(sock->id,kptr_null,as_okay,sock->error,NULL,
+		      NULL,NULL,NULL,NULL);
 		    sock->error = 0;
 		    /* @@@ clear call instead ? */
 		    new_state(sock,ss_connected);
@@ -557,22 +561,21 @@ void from_kernel(struct atmsvc_msg *msg,int size)
 	 */
 	return;
     }
-    if (msg->listen_vcc && (msg->type == as_accept || msg->type == as_reject ||
-      msg->type == as_identify)) {
+    if (kptr_eq(&msg->listen_vcc,&kptr_null) && (msg->type == as_accept ||
+      msg->type == as_reject || msg->type == as_identify)) {
 	dispatcher = dispatch_listen;
 	for (curr = sockets; curr; curr = curr->next)
-	    if (msg->listen_vcc == curr->id && (curr->state == ss_listening ||
-	      curr->state == ss_listen_zombie))
+	    if (kptr_eq(&msg->listen_vcc,&curr->id) &&
+	      (curr->state == ss_listening || curr->state == ss_listen_zombie))
 		break;
     }
     else {
 	dispatcher = dispatch;
 	for (curr = sockets; curr; curr = curr->next)
-	    if (msg->vcc == curr->id) break;
+	    if (kptr_eq(&msg->vcc,&curr->id)) break;
     }
-    diag(COMPONENT,DIAG_DEBUG,"FROM KERNEL: %s for socket %p (0x%lx/0x%lx) "
-      "in state %s",as_name[msg->type],curr,(unsigned long) msg->vcc,
-      (unsigned long) msg->listen_vcc,
-      state_name[curr ? curr->state : ss_null]);
+    diag(COMPONENT,DIAG_DEBUG,"FROM KERNEL: %s for socket %p (%s/%s) "
+      "in state %s",as_name[msg->type],curr,kptr_print(&msg->vcc),
+      kptr_print(&msg->listen_vcc),state_name[curr ? curr->state : ss_null]);
     dispatcher(curr,msg);
 }
