@@ -1,6 +1,6 @@
 /* uni.c - Processing of incoming UNI signaling messages */
  
-/* Written 1995-1999 by Werner Almesberger, EPFL-LRC/ICA */
+/* Written 1995-2000 by Werner Almesberger, EPFL-LRC/ICA */
  
 
 #include <stdarg.h>
@@ -9,10 +9,10 @@
 #include <string.h>
 #include <errno.h>
 #include <assert.h>
-#include <stdint.h> /* linux/atmdev.h includes linux/atm.h */
-#include <linux/atmdev.h>
 
 #include "atm.h"
+#include <linux/atmdev.h>
+
 #include "atmd.h"
 #include "uni.h"
 #include "qlib.h"
@@ -97,7 +97,7 @@ static void setup_call(SIG_ENTITY *sig,unsigned long call_ref)
 	send_release_complete(sig,call_ref,ATM_CV_REJ_CLIR);
 	return;
     }
-    this = new_sock(0);
+    this = new_sock(kptr_null);
     this->sig = sig;
     sock = lookup_sap(&in_addr,&in_sap,&in_qos,&this->local,&this->sap,
       &this->qos,0);
@@ -164,8 +164,8 @@ static void setup_call(SIG_ENTITY *sig,unsigned long call_ref)
 	    diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);
 	}
     }
-    send_kernel(0,sock->id,as_indicate,0,&this->pvc,&this->remote,&in_addr,
-      &this->sap,&this->qos);
+    send_kernel(kptr_null,sock->id,as_indicate,0,&this->pvc,&this->remote,
+      &in_addr,&this->sap,&this->qos);
     for (walk = &sock->listen; *walk; walk = &(*walk)->listen);
     *walk = this;
     diag(COMPONENT,DIAG_DEBUG,"SE vpi.vci=%d.%d",this->pvc.sap_addr.vpi,
@@ -380,8 +380,8 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    send_connect_ack(sock);
 	    /* @@@ fill in sock->remote */
 	    /* @@@ fill in traffic parameters */
-	    send_kernel(sock->id,0,as_okay,0,&sock->pvc,NULL,&sock->local,
-	      &sock->sap,&sock->qos);
+	    send_kernel(sock->id,kptr_null,as_okay,0,&sock->pvc,NULL,
+	      &sock->local,&sock->sap,&sock->qos);
 	    new_state(sock,ss_connected);
 #if defined(Q2963_1) || defined(DYNAMIC_UNI)
 	    sock->owner = 1;
@@ -389,8 +389,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *)
 	      &sock->remote,0) < 0) strcpy(buffer,"<invalid>");
 	    diag(COMPONENT,DIAG_INFO,"Active open succeeded (CR 0x%06X, "
-	      "ID 0x%08lx, to %s)",sock->call_ref,(unsigned long) sock->id,
-	      buffer);
+	      "ID %s, to %s)",sock->call_ref,kptr_print(&sock->id),buffer);
 	    return;
 	case ATM_MSG_CONN_ACK: /* ACCEPTING, WAIT_REL, REL_REQ */
 	    diag(COMPONENT,DIAG_DEBUG,"CA vpi.vci=%d.%d",
@@ -402,8 +401,8 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    }
 	    if (sock->state != ss_accepting) break;
 	    STOP_TIMER(sock);
-	    send_kernel(sock->id,0,as_okay,0,NULL,NULL,&sock->local,&sock->sap,
-	      NULL);
+	    send_kernel(sock->id,kptr_null,as_okay,0,NULL,NULL,&sock->local,
+	      &sock->sap,NULL);
 	    new_state(sock,ss_connected);
 #if defined(Q2963_1) || defined(DYNAMIC_UNI)
 	    sock->owner = 0;
@@ -411,8 +410,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1, (struct sockaddr *)
 	      &sock->remote,0) < 0) strcpy(buffer,"<invalid>");
 	    diag(COMPONENT,DIAG_INFO,"Passive open succeeded (CR 0x%06X, "
-	      "ID 0x%08lx, from %s)",sock->call_ref,(unsigned long) sock->id,
-	      buffer);
+	      "ID %s, from %s)",sock->call_ref,kptr_print(&sock->id),buffer);
 	    return;
 	case ATM_MSG_RELEASE: /* all states */
 	    {
@@ -584,8 +582,9 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 		sock->new_qos.rxtp.max_pcr = q_fetch(&in_dsc,QF_fw_pcr_01);
 	    if (q_present(&in_dsc,QF_bw_pcr_01))
 		sock->new_qos.txtp.max_pcr = q_fetch(&in_dsc,QF_bw_pcr_01);
-	    send_kernel(sock->id,0,as_modify,ATM_MF_INC_RSV | ATM_MF_DEC_RSV |
-	      ATM_MF_DEC_SHP,NULL,NULL,NULL,NULL,&sock->new_qos);
+	    send_kernel(sock->id,kptr_null,as_modify,
+	      ATM_MF_INC_RSV | ATM_MF_DEC_RSV | ATM_MF_DEC_SHP,
+	      NULL,NULL,NULL,NULL,&sock->new_qos);
 	    new_state(sock,ss_mod_rcv);
 	    return;
 	case ATM_MSG_MODIFY_ACK:
@@ -594,8 +593,8 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    STOP_TIMER(sock);
 	    sock->qos = sock->new_qos;
 	    if (q_present(&in_dsc,QG_bbrt)) send_conn_avail(sock);
-	    send_kernel(sock->id,0,as_modify,ATM_MF_SET,NULL,NULL,NULL,NULL,
-	      &sock->qos);
+	    send_kernel(sock->id,kptr_null,as_modify,ATM_MF_SET,NULL,NULL,NULL,
+	      NULL,&sock->qos);
 	    new_state(sock,ss_mod_fin_ok);
 	    return;
 	case ATM_MSG_MODIFY_REJ:
@@ -603,16 +602,16 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    if (sock->state != ss_mod_req) break;
 	    STOP_TIMER(sock);
 	    sock->error = -EAGAIN;
-	    send_kernel(sock->id,0,as_modify,ATM_MF_SET,NULL,NULL,NULL,NULL,
-	      &sock->qos);
+	    send_kernel(sock->id,kptr_null,as_modify,ATM_MF_SET,NULL,NULL,NULL,
+	      NULL,&sock->qos);
 	    new_state(sock,ss_mod_fin_fail);
 	    return;
 	case ATM_MSG_CONN_AVAIL:
 	    if (!(sock->sig->uni & S_Q2963_1)) goto _default;
 	    if (sock->state != ss_connected || sock->owner) break;
 	    STOP_TIMER(sock);
-	    send_kernel(sock->id,0,as_modify,ATM_MF_SET,NULL,NULL,NULL,NULL,
-	      &sock->qos);
+	    send_kernel(sock->id,kptr_null,as_modify,ATM_MF_SET,NULL,NULL,NULL,
+	      NULL,&sock->qos);
 	    new_state(sock,ss_mod_fin_ack);
 	    return;
 	_default: /* jump here if we don't want to understand a message */
@@ -656,10 +655,10 @@ void saal_failure(SIG_ENTITY *sig)
     trace_msg("SAAL went down");
     for (curr = sockets; curr; curr = next) {
 	next = curr->next;
-	if (curr->sig == sig && curr->call_state != cs_null)
-		if (curr->call_state != cs_active)
-		    uni_call(curr,ATM_MSG_RESTART);
-		else if (!t309) t309 = start_timer(T309_TIME,on_T309,sig);
+	if (curr->sig != sig || curr->call_state == cs_null) continue;
+	if (curr->call_state != cs_active)
+	    uni_call(curr,ATM_MSG_RESTART);
+	else if (!t309) t309 = start_timer(T309_TIME,on_T309,sig);
     }
 }
 
@@ -752,8 +751,8 @@ static void process_uni(SIG_ENTITY *sig,void *msg)
     for (curr = sockets; curr; curr = curr->next)
 	if (curr->sig == sig && curr->call_ref == call_ref) break;
     diag(COMPONENT,DIAG_DEBUG,"FROM SAAL %d.%d.%d: %s (0x%02X) CR 0x%06lx for "
-      "0x%lx",S_PVC(sig),mid2name(((unsigned char *) msg)[5]),
-      ((unsigned char *)msg)[5],call_ref,curr ? (unsigned long) curr->id : 0UL);
+      "%s",S_PVC(sig),mid2name(((unsigned char *) msg)[5]),
+      ((unsigned char *) msg)[5],call_ref,curr ? kptr_print(&curr->id) : "?");
     if (mid == ATM_MSG_SETUP) {
 	if (!curr) setup_call(sig,call_ref);
 	return;
@@ -773,11 +772,11 @@ static void process_uni(SIG_ENTITY *sig,void *msg)
 	}
     }
     if (!curr || curr->call_state == cs_null) {
-	if (mid != ATM_MSG_REL_COMP)
-	    if (mid != ATM_MSG_STATUS)
-		send_release_complete(sig,call_ref,ATM_CV_INV_CR);
-	    else if (q_fetch(&in_dsc,QF_call_state) != (int) cs_null)
-		    send_release_complete(sig,call_ref,ATM_CV_INCOMP_MSG);
+	if (mid == ATM_MSG_REL_COMP) return;
+	if (mid != ATM_MSG_STATUS)
+	    send_release_complete(sig,call_ref,ATM_CV_INV_CR);
+	else if (q_fetch(&in_dsc,QF_call_state) != (int) cs_null)
+		send_release_complete(sig,call_ref,ATM_CV_INCOMP_MSG);
 	return;
     }
     uni_call(curr,mid);
